function cueingTransfer2(subNo, justCond1, resume, trialsPerCond, maskContrast, targetRads, freq, justMax, preMaskFrms, shift, maskFrms, spread, checkLength)
% contrast4ID2(subNo, justCond1, resume, trialsPerCond, maskContrast, targetRads, freq, justMax, preMaskFrms, shift, maskFrms, spread, checkLength)
%
% justCond1       0 or 1 
% resume          0 or 1
% trialsPerCond   integer
% maskContrast    float between 0 and 1
% targetRads      float between 0 and pi/4
% freq            Nyquist is 39
% justMax         0 or 1
% preMaskFrms     non-negative ingeger
% shift           integer
% maskFrms        non-negative integer
% spread          positive integer
% checkLength     1 (lowest spectral density), 2, or 4 (highest)
%
% This one has 16 conditions:
% 1. single cue (same eye, valid)
% 1+i, 1<=i<=7. target i positions acw of single cue in the same eye
% 9. single cue (other eye, valid)
% 9+i, 1<=i<=7. target i positions acw of single cue in the other eye
%
% 01/03/2018  jas   Modified contrast4ID2.
% 
% e.g. cueingTransfer2(0, 0, 0, 1, 1, .2, 6.5, 0, 5, 168, 8, 36, 4)

headFrames=80; % Should be an even number.

exit=KbName('q'); 
ACW=KbName('c'); 
CW=KbName('m'); 

% files
datafilename = strcat('./cueingTransfer2_',num2str(subNo),'.dat'); % name of data file to write to
qfilename = strcat('./Q',num2str(subNo),'.mat'); % name of q file
datafilepointer = fopen(datafilename,'at'); % open ASCII file for writing

if justCond1==1
    nconds=1;
else
    nconds=16;
end
ntrials=nconds*trialsPerCond;
DxShuffle=zeros(1,ceil(ntrials/nconds))+1;
for i=2:nconds
    DxShuffle=Shuffle([DxShuffle zeros(1,ceil(ntrials/nconds))+i]);
end

% Set up QUEST. 
highestdB=0;
if justMax==1
    lowestdB=0;
else
    lowestdB=-60;
end

if resume==1
    load(qfilename)
else
    for i=1:nconds
        q(i)=QuestCreate(-30,30,.81,3.5,.02,.5,.2,60);
        q(i).normalizePdf=1; % This adds a few ms per call to QuestUpdate, but otherwise the pdf will underflow after about 1000 trials.
    end
end
    
% Startup checks:
PsychDefaultSetup(1);
sound(sin(1:375), 2000);
% Define screen to use:
whichScreen = max(Screen('Screens'));

% Open up a window on the screen for use in Bits++ mode on a
% CRS Bits#. This will also load an identity gamma
% ramp into the Bits device. Ask for stereomode 1 = frame-sequential
% stereo:
PsychImaging('PrepareConfiguration');
BitsPlusPlus('OpenBits#'); % Probably unnecessary.
PsychImaging('AddTask', 'General', 'EnableBits++Bits++Output');

% % crsDemoBitsPlusPlusSimple_modified_for_FEgoggles.m opens window with arguments
% % different to those below. Are the latter necessary for LUT animation?
% window = PsychImaging('OpenWindow', whichScreen, gray, [], [], [], 1); 
% Open the window, assign a background to index zero of the CLUT palette
[window, screenRect] = PsychImaging('OpenWindow', whichScreen, 0);
Priority(MaxPriority(window));       % added 06 Mar 2018

% Switch a Bits# display device into Bits++ mode.
BitsPlusPlus('SwitchToBits++');

% Enable FE1 stereo goggles connected to the Bits device:
BitsPlusPlus('UseFE1StereoGoggles', window, 1);

decoding_gamma = 1.745; %Changed 30 Jan 2018

blank=zeros(78);
blankTex=Screen('MakeTexture', window, blank);

fix=zeros(480);
for i=238:242
    for j=238:242
        fix(j,i)=255;
    end
end
for i=180:300
    fix(180,i)=255;fix(300,i)=255;fix(i,180)=255;fix(i,300)=255;
end
fixTex=Screen('MakeTexture', window, fix);

frmList1=zeros(480,480,8);
frmList2=zeros(480,480,8);
noiseFrame=zeros(480,480,8);
for k=1:8
    noiseFrame(:,:,k)=kron(round(rand(480/checkLength))+2*k-1,ones(checkLength));
end
for i=1:480
    for j=1:480
        if (194<=i&&i<=197 && 194<=j&&j<=287) || (194<=j&&j<=197 && 194<=i&&i<=287) ...
                || (194<=i&&i<=287 && 284<=j&&j<=287) || (194<=j&&j<=287 && 284<=i&&i<=287) 
            for k=1:8
                frmList1(i,j,k)=noiseFrame(i,j,k);
            end
        end
    end
end
for k=1:8
    frmList2(:,:,k)=circshift(frmList1(:,:,k), ...
        [mod(round(shift*sin(2*pi*(k-1)/8))-1,480)+1,mod(round(shift*cos(2*pi*(k-1)/8))-1,480)+1]);
end
frm8=frmList2(:,:,1)+frmList2(:,:,2)+frmList2(:,:,3)+frmList2(:,:,4)+ ...
    frmList2(:,:,5)+frmList2(:,:,6)+frmList2(:,:,7)+frmList2(:,:,8);
for i=238:242
    for j=238:242
        frm8(j,i)=255;
    end
end
for i=180:300
    frm8(180,i)=255;frm8(300,i)=255;frm8(i,180)=255;frm8(i,300)=255;
end
frmTex=Screen('MakeTexture', window, frm8);

mask=118*(2*round(rand(78))-1)+136;
maskTex=Screen('MakeTexture', window, mask);

lut1=repmat([.5;0;1;0;1;0;1;0;1;0;1;0;1;0;1;0;1;.5;zeros(254-18+1,1);0],1,3).^(1/decoding_gamma);
lut2=repmat([.5;0;1;0;1;0;1;0;1;0;1;0;1;0;1;0;1;.5;((maskContrast*(2*((18:254)-18)/(254-18)-1))/2+1/2)';0],1,3).^(1/decoding_gamma);

% loop through trials
for trial=1:ntrials
    repeatTrial=1;
    while repeatTrial==1
        cond=DxShuffle(trial);
        if justCond1==17
            tTest=2*round((QuestQuantile(q(1)))/2);
        else
            tTest=2*round((QuestQuantile(q(cond)))/2);
        end
        if tTest > highestdB
            tTest=highestdB;
        elseif tTest < lowestdB || isnan(tTest)
            tTest=lowestdB;
        end

        tmp=Shuffle(1:8);
        modCons=mod(cond-1,8)+1;
        nCues=1;
        cuedPosns=tmp(1:nCues);
        targPosn=mod(cuedPosns(1)+modCons-2,8)+1;
        vector3=[.5;0;1;0;1;0;1;0;1;0;1;0;1;0;1;0;1;.5;(((10^(tTest/20))*(2*((18:254)-18)/(254-18)-1))/2+1/2)';0];
        lut3=repmat(vector3,1,3).^(1/decoding_gamma);
        vector4=vector3;
        for i=1:length(cuedPosns)
            vector4(2*cuedPosns(i))=1;
            vector4(2*cuedPosns(i)+1)=0;
        end
        lut4=repmat(vector4,1,3).^(1/decoding_gamma);
        
        tiltSgn=2*round(rand)-1;
        phase=rand*2*pi;
        targ=round(118*GaborImage(78*[1,1],tiltSgn*targetRads*180/pi,freq/78,spread*[1,1]/sqrt(2*pi),phase)+136);
        targTex=Screen('MakeTexture', window, targ);
        targRect=Screen('Rect', targTex);
        targEye=2*round(rand)-1; %-1 is left; 1 is right.
               
        if 2*floor((cond-1)/8)-1==-1
            for frame = 1-headFrames:0
                % Schedule T-Lock for setting final shutter state at next flip:
                Screen('LoadNormalizedGammaTable', window, lut1, 2); 
                if floor(frame/2)==ceil(frame/2) % Surely there's a more efficient way of determining whether frame is odd or even!
                    Screen('DrawTexture', window, fixTex);
                    [~, Mask, Data, Command] = bitsGoggles((targEye+1)/2, (1-targEye)/2);
                else
                    Screen('DrawTexture', window, frmTex);
                    [~, Mask, Data, Command] = bitsGoggles((1-targEye)/2,(targEye+1)/2);
                end
                BitsPlusPlus('DIOCommand', window, 1, Mask, Data, Command);
                % Show them:
                Screen('Flip', window,[],1);
            end
            for frame = 1:2
                Screen('LoadNormalizedGammaTable', window, lut4, 2); 
                % Schedule T-Lock for setting final shutter state at next flip:
                if floor(frame/2)==ceil(frame/2) % Surely there's a more efficient way of determining whether frame is odd or even!
                    Screen('DrawTexture', window, fixTex);
                    [~, Mask, Data, Command] = bitsGoggles((targEye+1)/2, (1-targEye)/2);
                else
                    Screen('DrawTexture', window, frmTex);
                    [~, Mask, Data, Command] = bitsGoggles((1-targEye)/2,(targEye+1)/2);
                end
                BitsPlusPlus('DIOCommand', window, 1, Mask, Data, Command);
                % Show them:
                Screen('Flip', window,[],1);
            end
            for frame = 3:11
                Screen('LoadNormalizedGammaTable', window, lut3, 2); 
                % Schedule T-Lock for setting final shutter state at next flip:
                if floor(frame/2)==ceil(frame/2) % Surely there's a more efficient way of determining whether frame is odd or even!
                    Screen('DrawTexture', window, fixTex);
                    [~, Mask, Data, Command] = bitsGoggles((targEye+1)/2, (1-targEye)/2);
                else
                    Screen('DrawTexture', window, frmTex);
                    [~, Mask, Data, Command] = bitsGoggles((1-targEye)/2,(targEye+1)/2);
                end
                BitsPlusPlus('DIOCommand', window, 1, Mask, Data, Command);
                % Show them:
                Screen('Flip', window,[],1);
            end
            for frame = 12:15
                Screen('LoadNormalizedGammaTable', window, lut3, 2); 
                % Schedule T-Lock for setting final shutter state at next flip:
                if floor(frame/2)==ceil(frame/2) % Surely there's a more efficient way of determining whether frame is odd or even!
                    Screen('DrawTexture', window, fixTex);
                    [~, Mask, Data, Command] = bitsGoggles((targEye+1)/2, (1-targEye)/2);
                else
                    Screen('DrawTexture', window, frmTex);
                    Screen('DrawTexture', window, targTex,[],OffsetRect(CenterRect(targRect,screenRect),...
                        round(shift*cos(2*pi*(targPosn-1)/8)), round(shift*sin(2*pi*(targPosn-1)/8))));
                    [~, Mask, Data, Command] = bitsGoggles((1-targEye)/2,(targEye+1)/2);
                end
                BitsPlusPlus('DIOCommand', window, 1, Mask, Data, Command);
                % Show them:
                Screen('Flip', window,[],1);
             end
            for frame = 16:16+preMaskFrms
                Screen('LoadNormalizedGammaTable', window, lut1, 2); 
                % Schedule T-Lock for setting final shutter state at next flip:
                if floor(frame/2)==ceil(frame/2) % Surely there's a more efficient way of determining whether frame is odd or even!
                    Screen('DrawTexture', window, fixTex);
                    [~, Mask, Data, Command] = bitsGoggles((targEye+1)/2, (1-targEye)/2);
                else
                    Screen('DrawTexture', window, frmTex);
                    Screen('DrawTexture', window, blankTex,[],OffsetRect(CenterRect(targRect,screenRect),...
                        round(shift*cos(2*pi*(targPosn-1)/8)), round(shift*sin(2*pi*(targPosn-1)/8))));
                    [~, Mask, Data, Command] = bitsGoggles((1-targEye)/2,(targEye+1)/2);
                end
                BitsPlusPlus('DIOCommand', window, 1, Mask, Data, Command);
                % Show them:
                Screen('Flip', window,[],1);
            end
            for frame = 17+preMaskFrms:17+preMaskFrms+maskFrms
                Screen('LoadNormalizedGammaTable', window, lut2, 2); 
                % Schedule T-Lock for setting final shutter state at next flip:
                if floor(frame/2)==ceil(frame/2) % Surely there's a more efficient way of determining whether frame is odd or even!
                    Screen('DrawTexture', window, fixTex);
                    [~, Mask, Data, Command] = bitsGoggles((targEye+1)/2, (1-targEye)/2);
                else
                    Screen('DrawTexture', window, frmTex);
                    Screen('DrawTexture', window, maskTex,[],OffsetRect(CenterRect(targRect,screenRect),...
                        round(shift*cos(2*pi*(targPosn-1)/8)), round(shift*sin(2*pi*(targPosn-1)/8))));
                    [~, Mask, Data, Command] = bitsGoggles((1-targEye)/2,(targEye+1)/2);
                end
                BitsPlusPlus('DIOCommand', window, 1, Mask, Data, Command);
                % Show them:
                Screen('Flip', window,[],1);
            end
            while 1
                frame=frame+1;
                [KeyIsDown, ~, KeyCode]=KbCheck;
                if KeyIsDown
                    break;
                end
                Screen('LoadNormalizedGammaTable', window, lut1, 2); 
                % Schedule T-Lock for setting final shutter state at next flip:
                if floor(frame/2)==ceil(frame/2) % Surely there's a more efficient way of determining whether frame is odd or even!
                    Screen('DrawTexture', window, fixTex);
                    [~, Mask, Data, Command] = bitsGoggles((targEye+1)/2, (1-targEye)/2);
                else
                    Screen('DrawTexture', window, frmTex);
                    [~, Mask, Data, Command] = bitsGoggles((1-targEye)/2,(targEye+1)/2);
               end
                BitsPlusPlus('DIOCommand', window, 1, Mask, Data, Command);
                % Show them:
                Screen('Flip', window,[],1);
            end
        else
            for frame = 1-headFrames:0
                % Schedule T-Lock for setting final shutter state at next flip:
                Screen('LoadNormalizedGammaTable', window, lut1, 2); 
                if floor(frame/2)==ceil(frame/2) % Surely there's a more efficient way of determining whether frame is odd or even!
                    Screen('DrawTexture', window, frmTex);
                    [~, Mask, Data, Command] = bitsGoggles((targEye+1)/2, (1-targEye)/2);
                else
                    Screen('DrawTexture', window, fixTex);
                    [~, Mask, Data, Command] = bitsGoggles((1-targEye)/2,(targEye+1)/2);
                end
                BitsPlusPlus('DIOCommand', window, 1, Mask, Data, Command);
                % Show them:
                Screen('Flip', window,[],1);
            end
            for frame = 1:2
                Screen('LoadNormalizedGammaTable', window, lut4, 2); 
                % Schedule T-Lock for setting final shutter state at next flip:
                if floor(frame/2)==ceil(frame/2) % Surely there's a more efficient way of determining whether frame is odd or even!
                    Screen('DrawTexture', window, frmTex);
                    [~, Mask, Data, Command] = bitsGoggles((targEye+1)/2, (1-targEye)/2);
                else
                    Screen('DrawTexture', window, fixTex);
                    [~, Mask, Data, Command] = bitsGoggles((1-targEye)/2,(targEye+1)/2);
                end
                BitsPlusPlus('DIOCommand', window, 1, Mask, Data, Command);
                % Show them:
                Screen('Flip', window,[],1);
            end
            for frame = 3:13
                Screen('LoadNormalizedGammaTable', window, lut3, 2); 
                % Schedule T-Lock for setting final shutter state at next flip:
                if floor(frame/2)==ceil(frame/2) % Surely there's a more efficient way of determining whether frame is odd or even!
                    Screen('DrawTexture', window, frmTex);
                    [~, Mask, Data, Command] = bitsGoggles((targEye+1)/2, (1-targEye)/2);
                else
                    Screen('DrawTexture', window, fixTex);
                    [~, Mask, Data, Command] = bitsGoggles((1-targEye)/2,(targEye+1)/2);
                end
                BitsPlusPlus('DIOCommand', window, 1, Mask, Data, Command);
                % Show them:
                Screen('Flip', window,[],1);
            end
            for frame = 14:17
                Screen('LoadNormalizedGammaTable', window, lut3, 2); 
                % Schedule T-Lock for setting final shutter state at next flip:
                if floor(frame/2)==ceil(frame/2) % Surely there's a more efficient way of determining whether frame is odd or even!
                    Screen('DrawTexture', window, frmTex);
                    [~, Mask, Data, Command] = bitsGoggles((targEye+1)/2, (1-targEye)/2);
                else
                    Screen('DrawTexture', window, fixTex);
                    Screen('DrawTexture', window, targTex,[],OffsetRect(CenterRect(targRect,screenRect),...
                        round(shift*cos(2*pi*(targPosn-1)/8)), round(shift*sin(2*pi*(targPosn-1)/8))));
                    [~, Mask, Data, Command] = bitsGoggles((1-targEye)/2,(targEye+1)/2);
                end
                BitsPlusPlus('DIOCommand', window, 1, Mask, Data, Command);
                % Show them:
                Screen('Flip', window,[],1);
            end
            for frame = 18:18+preMaskFrms
                Screen('LoadNormalizedGammaTable', window, lut1, 2); 
                % Schedule T-Lock for setting final shutter state at next flip:
                if floor(frame/2)==ceil(frame/2) % Surely there's a more efficient way of determining whether frame is odd or even!
                    Screen('DrawTexture', window, frmTex);
                    [~, Mask, Data, Command] = bitsGoggles((targEye+1)/2, (1-targEye)/2);
                else
                    Screen('DrawTexture', window, fixTex);
                    Screen('DrawTexture', window, blankTex,[],OffsetRect(CenterRect(targRect,screenRect),...
                        round(shift*cos(2*pi*(targPosn-1)/8)), round(shift*sin(2*pi*(targPosn-1)/8))));
                    [~, Mask, Data, Command] = bitsGoggles((1-targEye)/2,(targEye+1)/2);
                end
                BitsPlusPlus('DIOCommand', window, 1, Mask, Data, Command);
                % Show them:
                Screen('Flip', window,[],1);
            end
            for frame = 19+preMaskFrms:19+preMaskFrms+maskFrms
                Screen('LoadNormalizedGammaTable', window, lut2, 2); 
                % Schedule T-Lock for setting final shutter state at next flip:
                if floor(frame/2)==ceil(frame/2) % Surely there's a more efficient way of determining whether frame is odd or even!
                    Screen('DrawTexture', window, frmTex);
                    [~, Mask, Data, Command] = bitsGoggles((targEye+1)/2, (1-targEye)/2);
                else
                    Screen('DrawTexture', window, fixTex);
                    Screen('DrawTexture', window, maskTex,[],OffsetRect(CenterRect(targRect,screenRect),...
                        round(shift*cos(2*pi*(targPosn-1)/8)), round(shift*sin(2*pi*(targPosn-1)/8))));
                    [~, Mask, Data, Command] = bitsGoggles((1-targEye)/2,(targEye+1)/2);
                end
                BitsPlusPlus('DIOCommand', window, 1, Mask, Data, Command);
                % Show them:
                Screen('Flip', window,[],1);
            end
            while 1
                frame=frame+1;
                [KeyIsDown, ~, KeyCode]=KbCheck;
                if KeyIsDown
                    break;
                end
                Screen('LoadNormalizedGammaTable', window, lut1, 2); 
                % Schedule T-Lock for setting final shutter state at next flip:
                if floor(frame/2)==ceil(frame/2) % Surely there's a more efficient way of determining whether frame is odd or even!
                    Screen('DrawTexture', window, frmTex);
                    [~, Mask, Data, Command] = bitsGoggles((targEye+1)/2, (1-targEye)/2);
                else
                    Screen('DrawTexture', window, fixTex);
                    [~, Mask, Data, Command] = bitsGoggles((1-targEye)/2,(targEye+1)/2);
                end
                BitsPlusPlus('DIOCommand', window, 1, Mask, Data, Command);
                % Show them:
                Screen('Flip', window,[],1);
            end
        end
        
        if KeyCode(exit)
            save(qfilename,'q')  % saving q structure.
            error('q pressed');
        elseif KeyCode(ACW)
            resp=1;
            repeatTrial=0;
        elseif KeyCode(CW)
            resp=2;
            repeatTrial=0;
       else 
            repeatTrial=1;
            % write message to subject
            Screen('DrawText', window, 'Neither c nor m was pressed. Click mouse to resume.', 10, 20, 255);
            Screen('Flip', window); % show text
            % wait for mouse press ( no GetClicks  :(  )
            buttons=0;
            while ~any(buttons) % wait for press
                [~,~,buttons] = GetMouse;
                WaitSecs(0.01);
            end
        end
        
        if repeatTrial==0
            if tiltSgn==1-2*(resp-1)
                response=0;
                sound(sin(1:375), 2000);
            else
                response=1;
            end
            % print to file
            fprintf(datafilepointer,'%03i %02i %+02i %+01i %+01i %01i %01i\n', ...
                trial, ...
                cond, ...
                tTest, ...
                tiltSgn, ...
                targEye, ...
                targPosn, ...
                resp);
            if justCond1==17
                q(1)=QuestUpdate(q(1),tTest,response); % Add the new datum (actual test intensity and observer response) to the database.
            else
                q(cond)=QuestUpdate(q(cond),tTest,response); % Add the new datum (actual test intensity and observer response) to the database.
            end
        end
        Screen('Close',targTex);
    end
end

% CLEAN UP.
% Release all textures:
Screen('Close');
% Disable FE1 stereo goggles connected to the Bits device:
BitsPlusPlus('UseFE1StereoGoggles', window, 0);
Screen('LoadNormalizedGammaTable', window, zeros(256,3), 2);
% This flip applies the LUT update and the DIOCommand scheduled above:
Screen('Flip', window);
% Release the connection to Bits#
BitsPlusPlus('Close');
% Close the window.
Screen('CloseAll');
save(qfilename,'q')  % saving q structure.
ShowCursor;
fclose('all');
Priority(0);    

% We are done.
return;
